#version 430

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout(location = 2) in vec2 uv;
layout (location = 3) in vec3 tangent;

out vec3 worldPosition;
out vec3 worldNormal;
out vec2 faceUv;
out vec4 FragPosLightSpace;
out mat3 TBN;

uniform mat4 projectionViewMat;
uniform mat4 modelMat;
uniform vec3 lightPosition;
uniform vec3 viewPosition;

uniform mat4 lightSpaceMatrix;


void main() {

  gl_Position= projectionViewMat * modelMat * vec4( position, 1.0 );
  worldNormal = vec3(normalize(modelMat * vec4(normal, 0.0)));
  worldPosition= vec3(modelMat* vec4(position, 1.0));
  faceUv= uv;
  FragPosLightSpace = lightSpaceMatrix * vec4(worldPosition, 1.0);

  vec3 T = normalize(vec3(modelMat * vec4(tangent, 0.0)));
  vec3 N = normalize(vec3(modelMat * vec4(normal, 0.0)));
  // re-orthogonalize T with respect to N (Gram-Schmidt) to make vectors perpendicular
  T = normalize(T - dot(T, N) * N);
  vec3 B = cross(N, T);

  TBN = mat3(T, B, N);

}
